import{f as o,p as e}from"./flowDb-c6c81e3f-C6yTYY1V.js";import{f as a,g as t}from"./styles-d45a18b0-BRU1Y9iQ.js";import{u as i}from"./mermaid.core-LXAjzBP7.js";import"./graph-CO6e1Y9R.js";import"./layout-CIAT9lqo.js";import"./index-5325376f-Bono_H96.js";import"./clone-BSKb7hJ3.js";import"./edges-96097737-BLVTVOT1.js";import"./createText-1719965b-DTHIXCt-.js";import"./line-3rPz4zTZ.js";import"./array-BKyUJesY.js";import"./path-CbwjOpE9.js";import"./channel-C_RmeJ5x.js";import"./index-BRuAXiMG.js";const n={parser:e,db:o,renderer:t,styles:a,init:r=>{r.flowchart||(r.flowchart={}),r.flowchart.arrowMarkerAbsolute=r.arrowMarkerAbsolute,i({flowchart:{arrowMarkerAbsolute:r.arrowMarkerAbsolute}}),t.setConf(r.flowchart),o.clear(),o.setGen("gen-2")}};export{n as diagram};
